// from http://n.mtng.org/ele/arduino/tutorial014.html

#include <SPI.h>
#include "Adafruit_MAX31855.h"
#include <Wire.h>
#include <FlexiTimer2.h>

#define CS   10
#define Tsc  1
#define Tin  0

Adafruit_MAX31855 thermocouple(CS);

// Connect INA226 with SCL (A3), SDA (A2) for arduino pro micro

#define SW_IN A2

int sample_trg = 0;

const byte INA226_ADDR = B1000000;
const byte INA226_CONFIG = 0x00;
const byte INA226_SHUNTV = 0x01;
const byte INA226_BUSV   = 0x02;
const byte INA226_POWER  = 0x03;
const byte INA226_CURRENT = 0x04;
const byte INA226_CALIB  = 0x05;
const byte INA226_MASK   = 0x06;
const byte INA226_ALERTL = 0x07;
const byte INA226_DIE_ID = 0xff;

void INA226_write(byte reg, unsigned short val)
{
  Wire.beginTransmission(INA226_ADDR);
  Wire.write(reg);
  Wire.write(val >> 8);
  Wire.write(val & 0xff);
  Wire.endTransmission();  
}

short INA226_read(byte reg)
{
  short ret = 0;
  // request the registor
  Wire.beginTransmission(INA226_ADDR);
  Wire.write(reg);
  Wire.endTransmission();  

  // read
  Wire.requestFrom((int)INA226_ADDR, 2);
  
  while(Wire.available()) {
    ret = (ret << 8) | Wire.read();
  }
  return ret;
}

double MAX31855_read(int com)
{
  if(com == Tin ){
    double c = thermocouple.readInternal();
    return c; 
  }
  else{
    double c = thermocouple.readCelsius();
    if (isnan(c)) {
      return 0xfff;
    } else {
      return c;   
    }
  }
}

void interrupt(){
  sample_trg = 1;
}


void setup() {
  Serial.begin(9600);
  while (!Serial) {}
  Serial.println("CURRENT & TEMPRATURE LOGGER");
  Serial.println("URUKA Technologies");
  Serial.println("2015/12/12");

  pinMode(SW_IN,INPUT_PULLUP);
  FlexiTimer2::set(100, 1.0/1000, interrupt); // call every 1ms "ticks"
  Wire.begin();
  // average: 16 times, conversion time: 8.244ms/8.244ms 
  INA226_write(INA226_CONFIG, 0x45ff);
  // current conversion
  INA226_write(INA226_CALIB,  2560);
  FlexiTimer2::start();
  Serial.println("setup end");
}

void loop() {
  int sv, bv, c;
  float bv_;
  double SCtmp,Rtmp;

  if ((sample_trg == 1) &&  ( digitalRead(SW_IN) == HIGH) ){
    sample_trg = 0;
    bv = INA226_read(INA226_BUSV);
    c = INA226_read(INA226_CURRENT);   
    bv_ *= 1.25;
    SCtmp = MAX31855_read(Tsc);
    Rtmp = MAX31855_read(Tin);
    Serial.print("V:");
    Serial.print(bv_);  // bus voltage in [mV]
    Serial.print(",A:");
    Serial.print(c);  // current in [mA]
    Serial.print(",T:"); // temprature in [celsius]
    Serial.print(SCtmp);
    Serial.print(",Tr:"); // temprature in [celsius]
    Serial.println(Rtmp);
  }
}
