#include <TinyGPS++.h>
#include <dht11.h>
#include "ESP8266.h"

// what digital pin we're connected to
#define DHTPIN 7     
// The DHT11 object
dht11 DHT11;

//GPS port
#define PIN_SERIAL3_RX 49  // D5
#define PIN_SERIAL3_TX 48  // D4
Uart Serial3(&sercom2, PIN_SERIAL3_RX, PIN_SERIAL3_TX);
static const uint32_t GPSBaud = 9600;
// The TinyGPS++ object
TinyGPSPlus gps;

//Network setting
#define SSID        "SSID"
#define PASSWORD    "PASSWORD"
#define HOST_NAME   "maker.ifttt.com"
#define HOST_PORT   (80)
#define EVENT         "EVENT"
#define SECRET_KEY    "SECRET_KEY"
// wifi object
ESP8266 wifi;

// Interrupt handler for SERCOM2
void SERCOM2_Handler()    
{
  Serial3.IrqHandler();
}

void setup() {
  Serial.begin(GPSBaud);
  Serial.println(" ");
  Serial.println("DHT11 & PA6C IFTTT LOGING TEST");
  Serial.println("start");
  Serial3.begin(GPSBaud);
  DHT11.attach(DHTPIN);
  wifi.begin(Serial5);
  Serial.print("FW Version:");
  Serial.println(wifi.getVersion().c_str());
  if (wifi.setOprToStationSoftAP()) {
    Serial.print("to station + softap ok\r\n");
  } else {
    Serial.print("to station + softap err\r\n");
  }
  if (wifi.joinAP(SSID, PASSWORD)) {
    Serial.print("Join AP success\r\n");
    Serial.print("IP:");
    Serial.println( wifi.getLocalIP().c_str());       
  } else {
    Serial.print("Join AP failure\r\n");
  }
  if (wifi.disableMUX()) {
    Serial.print("single ok\r\n");
  } else {
    Serial.print("single err\r\n");
  }  
  Serial.print("setup end\r\n");
}

void loop() {
  float gps_lat,gps_lng;
  int location_fix;
  int hum,temp;
  char buf1[128];
  char buf2[128];

  //get GPS data
  while(location_fix == 0){
    while (Serial3.available() > 0){
      if (gps.encode(Serial3.read())){
        gps_read(&gps_lat, &gps_lng, &location_fix);
      }
      if (millis() > 5000 && gps.charsProcessed() < 10) {
        location_fix = 0;
        Serial.println("No GPS detected: check wiring.");
        delay(500);
      }
    }
  }
  location_fix = 0;
  //debug GPS data
  Serial.print("LAT=");
  Serial.print(gps_lat);
  Serial.print(" LNG=");
  Serial.println(gps_lng);

  int lat_int = (int)gps_lat;
  long lat_p  = (long)(gps_lat*1000000);
  int lat_pp  = lat_p%1000000;
  int lng_int = (int)gps_lng;
  long lng_p  = (long)(gps_lng*1000000);
  int lng_pp  = lng_p%1000000;
 
  //get DHT data x10
  get_dht11(&hum,&temp);

  // IFTTT data send
  if (wifi.createTCP(HOST_NAME, HOST_PORT)) {
    Serial.println("Server connected");
    sprintf(buf1, "GET /trigger/%s/with/key/%s?value1=%d.%d&value2=%d.%06d&value3=%d.%06d HTTP/1.1\r\n", EVENT, SECRET_KEY, temp/10, temp%10, lat_int, lat_pp, lng_int, lng_pp);    
    sprintf(buf2, "Host: maker.ifttt.com\r\n");
    strcat(buf2, "Accept: */*\r\n");
    strcat(buf2, "\r\n");
    Serial.print(buf1);    
    Serial.print(buf2);    
    wifi.send((const uint8_t*)buf1, strlen(buf1));
    wifi.send((const uint8_t*)buf2, strlen(buf2));
    Serial.println("Request has sent!");
  } 
  delay(5000);
}

void gps_read(float *gps_lat, float *gps_lng, int *location_fix ){
  if (gps.location.isValid()) {
    *location_fix = 1;
    *gps_lat = gps.location.lat();
    *gps_lng = gps.location.lng();   
  }
  else {
    Serial.println("GPS INVALID");
    *location_fix = 0;
    *gps_lat = 0.0;
    *gps_lng = 0.0; 
   }
}

void get_dht11(int *hum, int *temp){
  int chk = DHT11.read();
  switch (chk)
  {
    case 0:  break;
    case -1: Serial.println("Checksum error"); break;
    case -2: Serial.println("Time out error"); break;
    default: Serial.println("Unknown error"); break;
  }
  float h = DHT11.humidity;
  // Read temperature as Celsius (the default)
  float t = DHT11.temperature;
  // Read temperature as Fahrenheit (isFahrenheit = true)
  *hum =(int)(h*10);
  *temp = (int)(t*10);    
}